﻿using System;
using System.IO;
using System.Windows.Forms;

// Számológép program, amely támogatja a négy alapműveltet valós számokkal
namespace ELTE.Forms.Calculator
{
    // a lehetséges műveleteket egy felsoroló típusban tároljuk
    public enum Operation { None, Add, Substract, Multiply, Divide }

    public partial class CalculatorForm : Form
    {
        private Double _FirstNumber, _SecondNumber; // eltároljuk az operandusokat
        private Operation _LastOperation; // eltároljuk a legutoljára használt műveletet

        public CalculatorForm()
        {
            InitializeComponent();
            _LastOperation = Operation.None; // kezdetben nincs művelet
            _FirstNumber = _SecondNumber = 0;
            _TextNumber.Text = "0";
        }

        // egy közös eseménykezelő az összes gombnak
        private void Button_Click(object sender, EventArgs e)
        {
            try
            {
                if (_LastOperation != Operation.None) // ha máűr volt művelet
                {
                    _SecondNumber = Double.Parse(_TextNumber.Text); // beolvassuk a második operandust
                    switch (_LastOperation) // végrehajtjuk a korábbi műveletet a két operandussal
                    {
                        case Operation.Add: Add(); break;
                        case Operation.Substract: Substract(); break;
                        case Operation.Multiply: Multiply(); break;
                        case Operation.Divide: Divide(); break;
                    }
                }
                // minden esetben:
                _FirstNumber = Double.Parse(_TextNumber.Text); // eltároljuk az első operandust
                switch (((Button)sender).Text)
                // megvizsgáljuk, milyen az eseményt kiváltó gomb felirata, így eldönthetjük, melyik gombot nyomták le
                {
                    case "+": _LastOperation = Operation.Add; break;
                    case "-": _LastOperation = Operation.Substract; break;
                    case "*": _LastOperation = Operation.Multiply; break;
                    case "/": _LastOperation = Operation.Divide; break;
                    case "=": _LastOperation = Operation.None; break; // beállítjuk a megfelelő műveletet
                }
            }
            catch (OverflowException)
            {
                MessageBox.Show("Your input has to many digits!", "Calculation Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (FormatException)
            {
                MessageBox.Show("Your input is not a real number!\nPlease correct!", "Calculation Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (NullReferenceException)
            {
                MessageBox.Show("No number in input!\nPlease correct!", "Calculation Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            finally
            {
                _TextNumber.Focus(); // visszaadjuk a vezérlést a szövegdoboznak
            }
        }

        // megfelelő művelet végrehajtása
        private void Add()
        {
            Double result = _FirstNumber + _SecondNumber; // művelet
            _ListHistory.Items.Add(_FirstNumber.ToString() + "+" + _SecondNumber.ToString() + "=" + result.ToString()); // művelet kiírása a listába
            _TextNumber.Text = result.ToString(); // eredmény kiírása a felső sávba
        }
        private void Substract()
        {
            Double result = _FirstNumber - _SecondNumber;
            _ListHistory.Items.Add(_FirstNumber.ToString() + "-" + _SecondNumber.ToString() + "=" + result.ToString());
            _TextNumber.Text = result.ToString();
        }
        private void Multiply()
        {
            Double result = _FirstNumber * _SecondNumber;
            _ListHistory.Items.Add(_FirstNumber.ToString() + "*" + _SecondNumber.ToString() + "=" + result.ToString());
            _TextNumber.Text = result.ToString();
        }
        private void Divide()
        {
            Double result = _FirstNumber / _SecondNumber;
            _ListHistory.Items.Add(_FirstNumber.ToString() + "/" + _SecondNumber.ToString() + "=" + result.ToString());
            _TextNumber.Text = result.ToString();
        }

        private void ButtonLoad_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog(); // fájl megnyitó dialógusablak 
            ofd.Title = "Open File"; // ablak címe
            if (ofd.ShowDialog() == DialogResult.OK) // ha az ablakot Ok-val zártuk be
            {
                StreamReader sr = new StreamReader(ofd.FileName);
                _TextNumber.Text = sr.ReadLine();
                _ListHistory.Items.Clear();
                while (!sr.EndOfStream)
                {
                    _ListHistory.Items.Add(sr.ReadLine());
                }
                sr.Close();
            }
        }

        private void ButtonSave_Click(object sender, EventArgs e)
        {
            SaveFileDialog sfd = new SaveFileDialog(); // fájl mentő dialógusablak
            sfd.Title = "Save to File";
            if (sfd.ShowDialog() == DialogResult.OK)
            {
                StreamWriter sw = new StreamWriter(sfd.FileName);
                sw.WriteLine(_TextNumber.Text); 
                foreach (String line in _ListHistory.Items) 
                {
                    sw.WriteLine(line);
                }
                sw.Close();
            }
        }
    }
}
